@@ -24,6 +24,7 @@ from osc_lib import exceptions
 from osc_lib.tests import fakes
 from osc_lib.tests import utils as test_utils
 from osc_lib import utils
+from osc_lib.utils import columns as column_utils
 
 PASSWORD = "Pa$$w0rd"
 WASSPORD = "Wa$$p0rd"
@@ -393,6 +394,81 @@ class TestUtils(test_utils.TestCase):
         self.assertIn('name', results)
         self.assertIn('size', results)
 
+    def _test_get_item_properties_with_formatter(self, formatters):
+        names = ('id', 'attr')
+        item = fakes.FakeResource(info={'id': 'fake-id', 'attr': ['a', 'b']})
+        res_id, res_attr = utils.get_item_properties(item, names,
+                                                     formatters=formatters)
+        self.assertEqual('fake-id', res_id)
+        return res_attr
+
+    def test_get_item_properties_with_format_func(self):
+        formatters = {'attr': utils.format_list}
+        res_attr = self._test_get_item_properties_with_formatter(formatters)
+        self.assertEqual(utils.format_list(['a', 'b']), res_attr)
+
+    def test_get_item_properties_with_formattable_column(self):
+        formatters = {'attr': format_columns.ListColumn}
+        res_attr = self._test_get_item_properties_with_formatter(formatters)
+        self.assertIsInstance(res_attr, format_columns.ListColumn)
+
+    def _test_get_dict_properties_with_formatter(self, formatters):
+        names = ('id', 'attr')
+        item = {'id': 'fake-id', 'attr': ['a', 'b']}
+        res_id, res_attr = utils.get_dict_properties(item, names,
+                                                     formatters=formatters)
+        self.assertEqual('fake-id', res_id)
+        return res_attr
+
+    def test_get_dict_properties_with_format_func(self):
+        formatters = {'attr': utils.format_list}
+        res_attr = self._test_get_dict_properties_with_formatter(formatters)
+        self.assertEqual(utils.format_list(['a', 'b']), res_attr)
+
+    def test_get_dict_properties_with_formattable_column(self):
+        formatters = {'attr': format_columns.ListColumn}
+        res_attr = self._test_get_dict_properties_with_formatter(formatters)
+        self.assertIsInstance(res_attr, format_columns.ListColumn)
+
+    def _test_calculate_header_and_attrs(self, parsed_args_columns,
+                                         expected_headers, expected_attrs):
+        column_headers = ('ID', 'Name', 'Fixed IP Addresses')
+        columns = ('id', 'name', 'fixed_ips')
+        parsed_args = mock.Mock()
+        parsed_args.columns = parsed_args_columns
+        ret_headers, ret_attrs = utils.calculate_header_and_attrs(
+            column_headers, columns, parsed_args)
+        self.assertEqual(expected_headers, ret_headers)
+        self.assertEqual(expected_attrs, ret_attrs)
+        if parsed_args_columns:
+            self.assertEqual(expected_headers, parsed_args.columns)
+        else:
+            self.assertFalse(parsed_args.columns)
+
+    def test_calculate_header_and_attrs_without_column_arg(self):
+        self._test_calculate_header_and_attrs(
+            [],
+            ('ID', 'Name', 'Fixed IP Addresses'),
+            ('id', 'name', 'fixed_ips'))
+
+    def test_calculate_header_and_attrs_with_known_columns(self):
+        self._test_calculate_header_and_attrs(
+            ['Name', 'ID'],
+            ['Name', 'ID'],
+            ['name', 'id'])
+
+    def test_calculate_header_and_attrs_with_unknown_columns(self):
+        self._test_calculate_header_and_attrs(
+            ['Name', 'ID', 'device_id'],
+            ['Name', 'ID', 'device_id'],
+            ['name', 'id', 'device_id'])
+
+    def test_calculate_header_and_attrs_with_attrname_columns(self):
+        self._test_calculate_header_and_attrs(
+            ['name', 'id', 'device_id'],
+            ['Name', 'ID', 'device_id'],
+            ['name', 'id', 'device_id'])
+
 
 class NoUniqueMatch(Exception):
     pass
@@ -632,6 +708,49 @@ class TestFindResource(test_utils.TestCase):
         self.assertEqual(expected, actual_unsorted)
 
 
+class TestColumnUtils(test_utils.TestCase):
+
+    def test_get_column_definitions(self):
+        attr_map = (
+            ('id', 'ID', column_utils.LIST_BOTH),
+            ('tenant_id', 'Project', column_utils.LIST_LONG_ONLY),
+            ('name', 'Name', column_utils.LIST_BOTH),
+            ('summary', 'Summary', column_utils.LIST_SHORT_ONLY),
+        )
+        headers, columns = column_utils.get_column_definitions(
+            attr_map, long_listing=False)
+        self.assertEqual(['id', 'name', 'summary'], columns)
+        self.assertEqual(['ID', 'Name', 'Summary'], headers)
+
+    def test_get_column_definitions_long(self):
+        attr_map = (
+            ('id', 'ID', column_utils.LIST_BOTH),
+            ('tenant_id', 'Project', column_utils.LIST_LONG_ONLY),
+            ('name', 'Name', column_utils.LIST_BOTH),
+            ('summary', 'Summary', column_utils.LIST_SHORT_ONLY),
+        )
+        headers, columns = column_utils.get_column_definitions(
+            attr_map, long_listing=True)
+        self.assertEqual(['id', 'tenant_id', 'name'], columns)
+        self.assertEqual(['ID', 'Project', 'Name'], headers)
+
+    def test_get_columns(self):
+        item = {
+            'id': 'test-id',
+            'tenant_id': 'test-tenant_id',
+            # 'name' is not included
+            'foo': 'bar',  # unknown attribute
+        }
+        attr_map = (
+            ('id', 'ID', column_utils.LIST_BOTH),
+            ('tenant_id', 'Project', column_utils.LIST_LONG_ONLY),
+            ('name', 'Name', column_utils.LIST_BOTH),
+        )
+        columns, display_names = column_utils.get_columns(item, attr_map)
+        self.assertEqual(tuple(['id', 'tenant_id', 'foo']), columns)
+        self.assertEqual(tuple(['ID', 'Project', 'foo']), display_names)
+
+
 class TestAssertItemEqual(test_utils.TestCommand):
 
     def test_assert_normal_item(self):
